<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">   
	<xsl:output method="xml" indent="yes" encoding="UTF-8"/>        
	
	<xsl:key name="package-by-subsystem" match="package" use="@subsystem" />
	
	<xsl:template match="radar-report">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()"/>
	      		<subsystems>
				<xsl:attribute name="nb-subsystems">
					<xsl:value-of select="count(packages/package[count(. | key('package-by-subsystem', @subsystem)[1]) = 1])"/>
				</xsl:attribute>
				<!-- Select each possible subsystem-->
				<xsl:for-each select="packages/package[count(. | key('package-by-subsystem', @subsystem)[1]) = 1]">
					<xsl:sort select="@subsystem"/>
					<xsl:variable name="subsystem-name" select="@subsystem"/>
					<subsystem>
						<xsl:attribute name="name"><xsl:value-of select="@subsystem"/></xsl:attribute>						
					</subsystem>
				</xsl:for-each>				
	      		</subsystems>
	   	</xsl:copy>
	</xsl:template>

	<xsl:template match="packages">
		<xsl:copy>
			<xsl:apply-templates select="@*"/>
	      	<xsl:for-each select="package">
	      		<xsl:sort select="@name"/>
	      		<xsl:copy>
				  	<xsl:apply-templates select="@*|node()"/>
				</xsl:copy>
	      	</xsl:for-each>
	   	</xsl:copy>
	</xsl:template>

    <xsl:template match="dependancy-metrics">
        <xsl:variable name="dependancy-metrics" select="."/>
  		<xsl:copy>
		  	<xsl:apply-templates select="@*|node()"/>
            <has-illegal-dependencies>
                <xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>
                    <xsl:call-template name="insert-value-element">
                            <xsl:with-param name="sequence-id" select="$current-seq-id"/>
                            <xsl:with-param name="value"
                                            select="count($dependancy-metrics/descendant::illegal-dependencies/value
                                                                    [
                                                                        (@seq-id=$current-seq-id)
                                                                            and
                                                                        (@value >  0)
                                                                    ])"/>
                        </xsl:call-template>
                </xsl:for-each>
            </has-illegal-dependencies>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="package-oo-metrics">
        <xsl:variable name="package-oo-metrics" select="."/>
  		<xsl:copy>
		  	<xsl:apply-templates select="@*|node()"/>
            <has-cycles>
                <xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>
                    <xsl:call-template name="insert-value-element">
                            <xsl:with-param name="value"
                                            select="count($package-oo-metrics/cycles/value
                                                                    [
                                                                        (@seq-id=$current-seq-id)
                                                                            and
                                                                        (@value='true')
                                                                    ])"/>
                            <xsl:with-param name="sequence-id" select="$current-seq-id"/>
                        </xsl:call-template>
                </xsl:for-each>
            </has-cycles>
        </xsl:copy>
    </xsl:template>

	<xsl:template match="source-metrics">
        <xsl:variable name="source-metrics" select="."/>
		<xsl:variable name="current-classes" select="../../classes"/>
  		<xsl:copy>
		  	<xsl:apply-templates select="@*|node()"/>
			<package-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

					<xsl:call-template name="insert-value-element">
						<xsl:with-param name="value"
										select="count($source-metrics/descendant::ncss/value
                                                                [
                                                                    (@seq-id != 1)
                                                                        and
                                                                    (@seq-id=$current-seq-id)
                                                                        and
                                                                    (@value != ../value[@seq-id=($current-seq-id - 1)]/@value)
                                                                ])"/>
						<xsl:with-param name="sequence-id" select="$current-seq-id"/>
					</xsl:call-template>
				</xsl:for-each>
			</package-changed>
            <classes-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

					<xsl:call-template name="insert-value-element">
						<xsl:with-param name="value"
										select="count($current-classes/descendant::class/descendant::ncss/value
                                                                [
                                                                    (@seq-id=$current-seq-id)
                                                                        and
                                                                    (@changed-since-last = 'true')
                                                                ])"/>
						<xsl:with-param name="sequence-id" select="$current-seq-id"/>
					</xsl:call-template>
				</xsl:for-each>
			</classes-changed>
			<methods-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

					<xsl:call-template name="insert-value-element">
						<xsl:with-param name="value"
										select="count($current-classes/descendant::ncss/value
															[
																(@seq-id=$current-seq-id)
																	and
																(@changed-since-last = 'true')
															])"/>

						<xsl:with-param name="sequence-id" select="$current-seq-id"/>
					</xsl:call-template>
				</xsl:for-each>
			</methods-changed>
			<source-statements-in-methods-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>
					<xsl:call-template name="insert-value-element">
						<xsl:with-param name="value"
										select="sum($current-classes/descendant::ncss/value
												[
													((@seq-id=$current-seq-id) and not(@value='NaN') and not(@value=''))
													and
													(@changed-since-last = 'true')
												]/@value)"/>
						<xsl:with-param name="sequence-id" select="$current-seq-id"/>
					</xsl:call-template>
				</xsl:for-each>
			</source-statements-in-methods-changed>
		</xsl:copy>
	</xsl:template>

   <xsl:template match="test-metrics">
		<xsl:variable name="current-classes" select="../../classes"/>
  		<xsl:copy>
		  	<xsl:apply-templates select="@*|node()"/>
			<coverage-on-methods-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

					<xsl:variable 	name="source-statements-covered-in-methods-changed"
									select="sum($current-classes/descendant::source-statements-covered/value
												[
													((@seq-id=$current-seq-id) and not(@value='NaN') and not(@value=''))
														and
													(../../ncss/value[@seq-id=$current-seq-id]/@changed-since-last = 'true')
												]/@value)"/>
					<xsl:variable 	name="source-statements-in-methods-changed"
								  	select="sum($current-classes/descendant::ncss/value
												[
													(@seq-id = $current-seq-id)
														and
													(@changed-since-last = 'true')
												]/@value)"/>
					<xsl:choose>
						<xsl:when test="$source-statements-in-methods-changed &gt; 0">
							<xsl:call-template name="insert-value-element">
								<xsl:with-param name="value" select="$source-statements-covered-in-methods-changed div $source-statements-in-methods-changed"/>
								<xsl:with-param name="sequence-id" select="$current-seq-id"/>
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="insert-value-element">
								<xsl:with-param name="value" select="'0'"/>
								<xsl:with-param name="sequence-id" select="$current-seq-id"/>
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</coverage-on-methods-changed>
			<source-statements-covered-on-methods-changed>
				<xsl:for-each select="/radar-report/reports/report">
	  				<xsl:variable name="current-seq-id" select="@seq-id"/>

					<xsl:call-template name="insert-value-element">
						<xsl:with-param name="value"
										select="sum($current-classes/descendant::source-statements-covered/value
												[
													((@seq-id=$current-seq-id) and not(@value='NaN') and not(@value=''))
														and
													(../../ncss/value[@seq-id=$current-seq-id]/@changed-since-last = 'true')
												]/@value)"/>
						<xsl:with-param name="sequence-id" select="$current-seq-id"/>
					</xsl:call-template>
				</xsl:for-each>
			</source-statements-covered-on-methods-changed>
		</xsl:copy>
	</xsl:template>

   <xsl:template name="insert-value-element">
		<xsl:param name="value"/>
		<xsl:param name="sequence-id"/>
		<value>
			<xsl:attribute name="seq-id"><xsl:value-of select="$sequence-id"/></xsl:attribute>
			<xsl:attribute name="value"><xsl:value-of select="$value"/></xsl:attribute>
		</value>
   </xsl:template>

   <!-- Copy all nodes from here. Can be overridden by a more specific XPath -->
	<xsl:template match="@*|node()">
		<xsl:copy>
	      	<xsl:apply-templates select="@*|node()"/>
	   	</xsl:copy>
	</xsl:template>


</xsl:stylesheet>
