<?xml version="1.0"  encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
  	extension-element-prefixes="redirect">


<xsl:template name="standard-analysis-source-control">
	<xsl:param name="reports-element"/>
	<xsl:param name="packages-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="single-package"/>
	<xsl:param name="package-element"/>

   	<xsl:variable name="file-identifier">
   		<xsl:choose>
 			<xsl:when test="$single-package='true'">
		 		<xsl:value-of select="$package-element/@name"/>
			</xsl:when>
	        <xsl:otherwise>
	            <xsl:value-of select="$subsystem-name"/>
	        </xsl:otherwise>
        </xsl:choose>
   </xsl:variable>

   <xsl:variable name="catalog-identifier">
   		<xsl:choose>
 			<xsl:when test="$single-package='true'">
		 		<xsl:value-of select="'packages'"/>
			</xsl:when>
	        <xsl:otherwise>
	            <xsl:value-of select="'subsystems'"/>
	        </xsl:otherwise>
        </xsl:choose>
   </xsl:variable>

	<h3>Source Control Metrics</h3>
	<p>
	    Source Control Metrics show the development activity for each release of the system. Activity for one class is the number of different
        issues that class was modified in. When aggregating to package or subsystem, these numbers are summed; i.e. a package where three classes
        were modified, in 1, 4 and 2 issues respectively, will report 1 + 4 + 2 = 7 as the activity number. Note, however, that several modifications
        on a class in one issue, counts as one activity.
    </p>
    <p>
        The issues are classified into two categories, change issues and error issues. Error percentage of total activity, will then be an
        indication on how much work is spent fixing bugs. Note that this measurement does not take "time spent" into account.
    </p>
    <p>
        The two graphs at the bottom shows the number of classes that were modified in change issues and error issues respectively. This metric
        does not take the number of changes on one class into account, thus for the example above, the metric would report 3 - since three classes
        were changed.
    </p>
    <table class="details">
	 	<tr><td style="background-color: white;">
		 		<div align="center">
	              <embed width="460" height="275" src="../img/source-control-total-activity-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
            <td style="background-color: white;"/>
            <td style="background-color: white;">

	            <div align="center">
	              <embed width="460" height="275" src="../img/source-control-error-percentage-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
	            </div>
            </td>
	 	</tr>
        <tr>
            <td style="background-color: white;">
                <div align="center">
                    <embed width="460" height="275" src="../img/source-control-change-classes-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
                </div>
            </td>
            <td style="background-color: white;"/>
            <td style="background-color: white;">

                <div align="center">
                    <embed width="460" height="275" src="../img/source-control-error-classes-image-{$file-identifier}.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
                </div>
            </td>
        </tr>




	 </table>

	<a href="javascript:winpopanalysisdetails('source-control-details-popup-{$file-identifier}.html')">[Source Control Metrics Details]</a><br/>

	<redirect:write file="{$docs-home}/{$catalog-identifier}/source-control-details-popup-{$file-identifier}.html">
	 <xsl:call-template name="standard-analysis-source-control-details">
		<xsl:with-param name="reports-element" select="$reports-element"/>
		<xsl:with-param name="packages-element" select="$packages-element"/>
		<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
		<xsl:with-param name="single-package" select="$single-package"/>
		<xsl:with-param name="package-element" select="$package-element"/>
	 </xsl:call-template>
	</redirect:write>

	<redirect:write file="{$docs-home}/img/source-control-total-activity-image-{$file-identifier}.svg">
        <xsl:call-template name="source-control-total-activity-graph">
			<xsl:with-param name="reports-element" select="$reports-element"/>
			<xsl:with-param name="packages-element" select="$packages-element"/>
			<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
			<xsl:with-param name="single-package" select="$single-package"/>
			<xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

	<redirect:write file="{$docs-home}/img/source-control-error-percentage-image-{$file-identifier}.svg">
		 <xsl:call-template name="source-control-error-percentage-graph">
             <xsl:with-param name="reports-element" select="$reports-element"/>
             <xsl:with-param name="packages-element" select="$packages-element"/>
             <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
             <xsl:with-param name="single-package" select="$single-package"/>
             <xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

    <redirect:write file="{$docs-home}/img/source-control-error-classes-image-{$file-identifier}.svg">
		 <xsl:call-template name="standard-dynamics-line-graph">
             <xsl:with-param name="text-y-axis" select="'Classes with Errors'"/>
             <xsl:with-param name="primary-metric-group" select="'source-control-metrics'"/>
             <xsl:with-param name="primary-metric" select="'error-classes'"/>
             <xsl:with-param name="y-axis-justification" select="'simple-normalize'"/>

             <xsl:with-param name="reports-element" select="$reports-element"/>
             <xsl:with-param name="packages-element" select="$packages-element"/>
             <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
             <xsl:with-param name="single-package" select="$single-package"/>
             <xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

    <redirect:write file="{$docs-home}/img/source-control-change-classes-image-{$file-identifier}.svg">
		 <xsl:call-template name="standard-dynamics-line-graph">
             <xsl:with-param name="text-y-axis" select="'Classes with Changes'"/>
             <xsl:with-param name="primary-metric-group" select="'source-control-metrics'"/>
             <xsl:with-param name="primary-metric" select="'change-classes'"/>
             <xsl:with-param name="y-axis-justification" select="'simple-normalize'"/>

             <xsl:with-param name="reports-element" select="$reports-element"/>
             <xsl:with-param name="packages-element" select="$packages-element"/>
             <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
             <xsl:with-param name="single-package" select="$single-package"/>
             <xsl:with-param name="package-element" select="$package-element"/>
		 </xsl:call-template>
	</redirect:write>

</xsl:template>

<xsl:template name="standard-analysis-source-control-details">
	<xsl:param name="reports-element"/>
	<xsl:param name="packages-element"/>
	<xsl:param name="subsystem-name"/>
	<xsl:param name="single-package"/>
	<xsl:param name="package-element"/>
	<html>
	<head>
         <link rel="stylesheet" type="text/css" href="../stylesheet.css"/>
	</head>
    <body>

	<table class="details">
		<tr><th>Version</th><th>Date</th><th>Packages</th><th>Classes</th><th>Abstract Cl.</th><th>Concrete Cl.</th><th>Methods</th>
            <th>Classes with Changes</th><th>Classes with Errors</th>
            <th colspan="2">Change Activity</th><th colspan="2">Error Activity</th><th>Total Activity</th>
        </tr>
		<xsl:for-each select="$reports-element/report">
			<xsl:sort select="@seq-id"/>
			<xsl:variable name="current-seq-id" select="@seq-id"/>

			<xsl:choose>
				<xsl:when test="$single-package='true'">
			 		<xsl:if test="$package-element/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]">
				 		<xsl:variable name="classes" select="$package-element/statistics/source-control-metrics/total-classes/value[@seq-id=$current-seq-id]/@value"/>
						<xsl:variable name="methods" select="$package-element/statistics/source-metrics/functions/value[@seq-id=$current-seq-id]/@value"/>
						<xsl:variable name="packages" select="count($package-element/package-availability/report[@seq-id=$current-seq-id])"/>
						<xsl:variable name="packages-changed"
							select="count($package-element/package-availability/report[
								(@seq-id=$current-seq-id) and
									(../../statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value != ../../statistics/source-metrics/ncss/value[@seq-id=($current-seq-id - 1)]/@value)
								])"/>

                         <xsl:variable name="change-activity" select="$packages-element/statistics/source-control-metrics/change-activity/value[@seq-id=$current-seq-id]/@value"/>
                         <xsl:variable name="error-activity" select="$packages-element/statistics/source-control-metrics/error-activity/value[@seq-id=$current-seq-id]/@value"/>
                         <xsl:variable name="change-classes" select="$packages-element/statistics/source-control-metrics/change-classes/value[@seq-id=$current-seq-id]/@value"/>
                         <xsl:variable name="error-classes" select="$packages-element/statistics/source-control-metrics/error-classes/value[@seq-id=$current-seq-id]/@value"/>
                         <xsl:variable name="change-percentage">
                             <xsl:choose>
                                 <xsl:when test="$change-activity=0">0</xsl:when>
                                 <xsl:otherwise><xsl:value-of select="100 * $change-activity div ($change-activity + $error-activity)"/></xsl:otherwise>
                             </xsl:choose>
                         </xsl:variable>
                         <xsl:variable name="error-percentage">
                             <xsl:choose>
                                 <xsl:when test="$error-activity=0">0</xsl:when>
                                 <xsl:otherwise><xsl:value-of select="100 * $error-activity div ($change-activity + $error-activity)"/></xsl:otherwise>
                             </xsl:choose>
                         </xsl:variable>

				 		<tr>
                             <td><xsl:value-of select="version"/></td>
                             <td><xsl:value-of select="date"/></td>
                             <td><xsl:value-of select="$packages"/></td>
                             <td><xsl:value-of select="$classes"/></td>
                             <td><xsl:value-of select="$package-element/statistics/source-metrics/abstract-classes/value[@seq-id=$current-seq-id]/@value"/></td>
                             <td><xsl:value-of select="$package-element/statistics/source-metrics/concrete-classes/value[@seq-id=$current-seq-id]/@value"/></td>
                             <td><xsl:value-of select="$methods"/></td>
                             <td><xsl:value-of select="$change-classes"/></td>
                             <td><xsl:value-of select="$error-classes"/></td>
                             <td><xsl:value-of select="$change-activity"/></td>
                             <td align="right"><xsl:value-of select="format-number($change-percentage, '#.##')"/>%</td>
                             <td><xsl:value-of select="$error-activity"/></td>
                             <td align="right"><xsl:value-of select="format-number($error-percentage, '#.##')"/>%</td>
                             <td><xsl:value-of select="$change-activity + $error-activity"/></td>
						</tr>
					</xsl:if>
				</xsl:when>
		        <xsl:otherwise>
				    <xsl:if test="$packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]">
					    <xsl:variable name="classes" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/total-classes/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="methods" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/functions/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="ncss" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="packages" select="count($packages-element/package[starts-with(@subsystem,$subsystem-name) and (statistics/source-metrics)]/package-availability/report[@seq-id=$current-seq-id])"/>

						<xsl:variable name="packages-changed"
							select="count($packages-element/package[starts-with(@subsystem,$subsystem-name) and (statistics/source-metrics)]/package-availability/report[
								(@seq-id=$current-seq-id) and
									(../../statistics/source-metrics/ncss/value[@seq-id=$current-seq-id]/@value != ../../statistics/source-metrics/ncss/value[@seq-id=($current-seq-id - 1)]/@value)

								])"/>
                        <xsl:variable name="change-activity" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/change-activity/value[@seq-id=$current-seq-id]/@value)"/>
						<xsl:variable name="error-activity" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/error-activity/value[@seq-id=$current-seq-id]/@value)"/>
                        <xsl:variable name="change-classes" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/change-classes/value[@seq-id=$current-seq-id]/@value)"/>
                        <xsl:variable name="error-classes" select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/error-classes/value[@seq-id=$current-seq-id]/@value)"/>
                        <xsl:variable name="change-percentage">
                            <xsl:choose>
                                <xsl:when test="$change-activity=0">0</xsl:when>
                                <xsl:otherwise><xsl:value-of select="100 * $change-activity div ($change-activity + $error-activity)"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:variable name="error-percentage">
                            <xsl:choose>
                                <xsl:when test="$error-activity=0">0</xsl:when>
                                <xsl:otherwise><xsl:value-of select="100 * $error-activity div ($change-activity + $error-activity)"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>

						<tr>
							<td><xsl:value-of select="version"/></td>
							<td><xsl:value-of select="date"/></td>
							<td><xsl:value-of select="$packages"/></td>
							<td><xsl:value-of select="$classes"/></td>
							<td><xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/abstract-classes/value[@seq-id=$current-seq-id]/@value)"/></td>
							<td><xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-metrics/concrete-classes/value[@seq-id=$current-seq-id]/@value)"/></td>
							<td><xsl:value-of select="$methods"/></td>
                            <td><xsl:value-of select="$change-classes"/></td>
                            <td><xsl:value-of select="$error-classes"/></td>
							<td><xsl:value-of select="$change-activity"/></td>
                            <td align="right"><xsl:value-of select="format-number($change-percentage, '0.00')"/>%</td>
                            <td><xsl:value-of select="$error-activity"/></td>
                            <td align="right"><xsl:value-of select="format-number($error-percentage, '0.00')"/>%</td>
							<td><xsl:value-of select="$change-activity + $error-activity"/></td>
						</tr>
					</xsl:if>
		        </xsl:otherwise>
		    </xsl:choose>

		</xsl:for-each>
	</table>

	</body>
	</html>

</xsl:template>


<xsl:template name="source-control-total-activity-graph">
    <xsl:param name="reports-element"/>
    <xsl:param name="packages-element"/>
    <xsl:param name="subsystem-name"/>
    <xsl:param name="single-package"/>
    <xsl:param name="package-element"/>

    <xsl:call-template name="line-graph">
          <xsl:with-param name="text-x-axis" select="'Version'"/>
          <xsl:with-param name="text-y-axis" select="'Total Activity'"/>
          <xsl:with-param name="width" select="'1300'"/>
          <xsl:with-param name="height" select="'800'"/>
          <xsl:with-param name="point-nodeset">
              <points>
                  <xsl:for-each select="$reports-element/report">
                      <xsl:sort select="@seq-id"/>
                      <xsl:variable name="current-seq-id" select="@seq-id"/>
                      <xsl:variable name="total-activity">
                          <xsl:choose>
                              <xsl:when test="$single-package='true'">
                                  <xsl:value-of select="sum($package-element/statistics/source-control-metrics/change-activity/value[@seq-id = $current-seq-id]/@value)
                                                      + sum($package-element/statistics/source-control-metrics/error-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:when>
                              <xsl:otherwise>
                                  <xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/change-activity/value[@seq-id = $current-seq-id]/@value)
                                                      + sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/error-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:otherwise>
                          </xsl:choose>
                      </xsl:variable>
                      <point>
                          <x-label><xsl:value-of select="version"/></x-label>
                          <x-coordinate><xsl:value-of select="sum(../report[@seq-id &lt;= $current-seq-id]/days-since-last)"/></x-coordinate>
                          <y-coordinate><xsl:value-of select="$total-activity"/></y-coordinate>
                      </point>
                  </xsl:for-each>
              </points>
          </xsl:with-param>
    </xsl:call-template>

</xsl:template>

<xsl:template name="source-control-error-percentage-graph">
    <xsl:param name="reports-element"/>
    <xsl:param name="packages-element"/>
    <xsl:param name="subsystem-name"/>
    <xsl:param name="single-package"/>
    <xsl:param name="package-element"/>

    <xsl:call-template name="line-graph">
          <xsl:with-param name="text-x-axis" select="'Version'"/>
          <xsl:with-param name="text-y-axis" select="'Error % of Total Activity'"/>
          <xsl:with-param name="width" select="'1300'"/>
          <xsl:with-param name="height" select="'800'"/>
          <xsl:with-param name="point-nodeset">
              <points>
                  <xsl:for-each select="$reports-element/report">
                      <xsl:sort select="@seq-id"/>
                      <xsl:variable name="current-seq-id" select="@seq-id"/>
                      <xsl:variable name="change-activity">
                          <xsl:choose>
                              <xsl:when test="$single-package='true'">
                                  <xsl:value-of select="sum($package-element/statistics/source-control-metrics/change-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:when>
                              <xsl:otherwise>
                                  <xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/change-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:otherwise>
                          </xsl:choose>
                      </xsl:variable>
                      <xsl:variable name="error-activity">
                          <xsl:choose>
                              <xsl:when test="$single-package='true'">
                                  <xsl:value-of select="sum($package-element/statistics/source-control-metrics/error-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:when>
                              <xsl:otherwise>
                                  <xsl:value-of select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/source-control-metrics/error-activity/value[@seq-id = $current-seq-id]/@value)"/>
                              </xsl:otherwise>
                          </xsl:choose>
                      </xsl:variable>
                      <xsl:variable name="error-percentage">
                          <xsl:choose>
                              <xsl:when test="$error-activity=0">0</xsl:when>
                              <xsl:otherwise>
                                  <xsl:value-of select="100 * $error-activity div ($error-activity + $change-activity)"/>
                              </xsl:otherwise>
                          </xsl:choose>
                      </xsl:variable>
                      <point>
                          <x-label><xsl:value-of select="version"/></x-label>
                          <x-coordinate><xsl:value-of select="sum(../report[@seq-id &lt;= $current-seq-id]/days-since-last)"/></x-coordinate>
                          <y-coordinate><xsl:value-of select="$error-percentage"/></y-coordinate>
                      </point>
                  </xsl:for-each>
              </points>
          </xsl:with-param>
    </xsl:call-template>

</xsl:template>

</xsl:stylesheet>
