<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan">

<xsl:include href="common-scripts.xsl"/>
<xsl:include href="common-graph.xsl"/>


<xsl:variable name="main-graph-line-color" select="'$color-ko'"/>
<xsl:variable name="grid-point-text-color" select="$axis-color"/>

<xsl:template name="line-graph">
  <xsl:param name="text-x-axis"/>
  <xsl:param name="text-y-axis"/>
  <xsl:param name="width"/>
  <xsl:param name="height"/>
  <xsl:param name="point-nodeset"/>
  <xsl:param name="y-axis-justification"/>

	<svg viewBox="0 0 {$width} {$height}" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">

		<xsl:variable name="pointset" select="xalan:nodeset($point-nodeset)"/>



        <xsl:call-template name="common-scripts"/>

		<!-- Include common features -->
		<xsl:call-template name="common-graph">
		  	<xsl:with-param name="text-x-axis" select="$text-x-axis"/>
		  	<xsl:with-param name="text-y-axis" select="$text-y-axis"/>
		  	<xsl:with-param name="width" select="$width"/>
			<xsl:with-param name="height" select="$height"/>
		</xsl:call-template>

        <xsl:call-template name="draw-aggregate">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="pointset" select="$pointset"/>
            <xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
        </xsl:call-template>

		<xsl:if test="$pointset/subcomponents">
        <xsl:call-template name="draw-subcomponents">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="pointset" select="$pointset"/>
            <xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
        </xsl:call-template>
		</xsl:if>



	</svg>

</xsl:template>

<xsl:template name="draw-aggregate">
    <xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="pointset"/>
	<xsl:param name="y-axis-justification"/>

    <xsl:variable name="max-x">
        <xsl:for-each select="$pointset/points/point">
            <xsl:sort select="x-coordinate" order="descending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="x-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="min-x">
        <xsl:for-each select="$pointset/points/point">
            <xsl:sort select="x-coordinate" order="ascending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="x-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="max-y">
        <xsl:for-each select="$pointset/points/point">
            <xsl:sort select="y-coordinate" order="descending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="y-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="min-y">
        <xsl:for-each select="$pointset/points/point">
            <xsl:sort select="y-coordinate" order="ascending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="y-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <g style="visibility:visible;" id="aggregate">

		<xsl:if test="$pointset/subcomponents">
        <g style="visibility:visible;"
            id="subcomponent-link"
            onclick="onEvent(evt, 'aggregate', 'hidden'); onEvent(evt, 'subcomponent-link', 'hidden'); onEvent(evt, 'subcomponents', 'visible'); onEvent(evt, 'aggregate-link', 'visible');"
            onmouseover="onEvent(evt, 'subcomponent-link-highlight', 'visible');"
            onmouseout="onEvent(evt, 'subcomponent-link-highlight', 'hidden');">

            <xsl:call-template name="draw-link-box">
                <xsl:with-param name="width" select="$width"/>
                <xsl:with-param name="height" select="$height"/>
                <xsl:with-param name="text-line1" select="'Subsystem'"/>
                <xsl:with-param name="text-line2" select="'Measures   -->'"/>
            </xsl:call-template>

        </g>

        <g style="visibility:hidden;" id="subcomponent-link-highlight">
            <rect
                x="{$width * 0.79}"
                y="{$height * (0.02)}"
                fill="none"
                stroke="red"
                stroke-width="1">
                <xsl:attribute name="width"><xsl:value-of select="ceiling($width * 0.13)"/></xsl:attribute>
                <xsl:attribute name="height"><xsl:value-of select="ceiling($height * 0.08)"/></xsl:attribute>
            </rect>
        </g>
		</xsl:if>

        <xsl:call-template name="draw-poly-line">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="pointset" select="$pointset"/>
            <xsl:with-param name="max-x" select="$max-x"/>
            <xsl:with-param name="min-x" select="$min-x"/>
            <xsl:with-param name="max-y" select="$max-y"/>
            <xsl:with-param name="min-y" select="$min-y"/>
            <xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
        </xsl:call-template>

        <xsl:call-template name="draw-y-axis">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="max-x" select="$max-x"/>
            <xsl:with-param name="min-x" select="$min-x"/>
            <xsl:with-param name="max-y" select="$max-y"/>
            <xsl:with-param name="min-y" select="$min-y"/>
            <xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
        </xsl:call-template>

        <xsl:call-template name="draw-x-axis">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="pointset" select="$pointset"/>
            <xsl:with-param name="max-x" select="$max-x"/>
            <xsl:with-param name="min-x" select="$min-x"/>
            <xsl:with-param name="max-y" select="$max-y"/>
            <xsl:with-param name="min-y" select="$min-y"/>
            <xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
        </xsl:call-template>
    </g>

</xsl:template>

<xsl:template name="draw-subcomponents">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="pointset"/>
    <xsl:param name="y-axis-justification"/>

	<xsl:variable name="max-x">
        <xsl:for-each select="$pointset/subcomponents/component/points/point">
            <xsl:sort select="x-coordinate" order="descending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="x-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="min-x">
        <xsl:for-each select="$pointset/subcomponents/component/points/point">
            <xsl:sort select="x-coordinate" order="ascending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="x-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="max-y">
        <xsl:for-each select="$pointset/subcomponents/component/points/point">
            <xsl:sort select="y-coordinate" order="descending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="y-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="min-y">
        <xsl:for-each select="$pointset/subcomponents/component/points/point">
            <xsl:sort select="y-coordinate" order="ascending" data-type="number"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="y-coordinate"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>

    <g style="visibility:hidden;" id="subcomponents">
        <g style="visibility:hidden;"
            id="aggregate-link"
            onclick="onEvent(evt, 'subcomponents', 'hidden'); onEvent(evt, 'aggregate-link', 'hidden'); onEvent(evt, 'aggregate', 'visible'); onEvent(evt, 'subcomponent-link', 'visible');"
            onmouseover="onEvent(evt, 'aggregate-link-highlight', 'visible');"
            onmouseout="onEvent(evt, 'aggregate-link-highlight', 'hidden');">

            <xsl:call-template name="draw-link-box">
                <xsl:with-param name="width" select="$width"/>
                <xsl:with-param name="height" select="$height"/>
                <xsl:with-param name="text-line1" select="'Aggregate'"/>
                <xsl:with-param name="text-line2" select="'Measure    -->'"/>
            </xsl:call-template>

        </g>

        <g style="visibility:hidden;" id="aggregate-link-highlight">
            <rect
                x="{$width * 0.79}"
                y="{$height * (0.02)}"
                fill="none"
                stroke="red"
                stroke-width="1">
                <xsl:attribute name="width"><xsl:value-of select="ceiling($width * 0.13)"/></xsl:attribute>
                <xsl:attribute name="height"><xsl:value-of select="ceiling($height * 0.08)"/></xsl:attribute>
            </rect>
        </g>

		<xsl:call-template name="draw-y-axis">
			<xsl:with-param name="width" select="$width"/>
			<xsl:with-param name="height" select="$height"/>
			<xsl:with-param name="max-x" select="$max-x"/>
			<xsl:with-param name="min-x" select="$min-x"/>
			<xsl:with-param name="max-y" select="$max-y"/>
			<xsl:with-param name="min-y" select="$min-y"/>
			<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		</xsl:call-template>

		<xsl:call-template name="draw-x-axis">
			<xsl:with-param name="width" select="$width"/>
			<xsl:with-param name="height" select="$height"/>
			<xsl:with-param name="pointset" select="$pointset"/>
			<xsl:with-param name="max-x" select="$max-x"/>
			<xsl:with-param name="min-x" select="$min-x"/>
			<xsl:with-param name="max-y" select="$max-y"/>
			<xsl:with-param name="min-y" select="$min-y"/>
			<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		</xsl:call-template>


		<xsl:for-each select="$pointset/subcomponents/component">
			<g id="{@name}-dash"
				onmouseover="onEvent(evt, '{@name}-full', 'visible');"
				onmouseout="onEvent(evt, '{@name}-full', 'hidden');">
				<xsl:call-template name="draw-poly-line">
					<xsl:with-param name="width" select="$width"/>
					<xsl:with-param name="height" select="$height"/>
					<xsl:with-param name="pointset" select="current()"/>
					<xsl:with-param name="max-x" select="$max-x"/>
					<xsl:with-param name="min-x" select="$min-x"/>
					<xsl:with-param name="max-y" select="$max-y"/>
					<xsl:with-param name="min-y" select="$min-y"/>
					<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
					<xsl:with-param name="style" select="'stroke-dasharray:10,20'"/>
				</xsl:call-template>
			</g>

			<g style="visibility:hidden;"
				id="{@name}-full"
				onmouseover="onEvent(evt, '{@name}-full', 'visible');"
				onmouseout="onEvent(evt, '{@name}-full', 'hidden');">
				<xsl:call-template name="draw-poly-line">
					<xsl:with-param name="width" select="$width"/>
					<xsl:with-param name="height" select="$height"/>
					<xsl:with-param name="pointset" select="current()"/>
					<xsl:with-param name="max-x" select="$max-x"/>
					<xsl:with-param name="min-x" select="$min-x"/>
					<xsl:with-param name="max-y" select="$max-y"/>
					<xsl:with-param name="min-y" select="$min-y"/>
					<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
				</xsl:call-template>
				<text font-family="Verdana"
					fill="$canvas-color"
					x="{$width * 0.12}"
			        y="{$height * (0.12)}"
					font-size="{ceiling($width * 0.02)}">
					<xsl:value-of select="@name"/>
				</text>
			</g>
		</xsl:for-each>

    </g>

</xsl:template>



<!-- plots a polyline on the graph -->
<xsl:template name="draw-poly-line">
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="pointset"/>
	<xsl:param name="max-x"/>
	<xsl:param name="min-x"/>
	<xsl:param name="max-y"/>
	<xsl:param name="min-y"/>
	<xsl:param name="y-axis-justification"/>
	<xsl:param name="style" select="''"/>


	<xsl:variable name="points-series">
		<xsl:for-each select="$pointset/points/point">
			<xsl:sort data-type="number" select="x-coordinate"/>
			<xsl:choose>
				<xsl:when test="$y-axis-justification='simple-normalize'">
					<xsl:choose>
						<xsl:when test="$max-y = $min-y">
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.5) ),
								' ')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.9 - 0.8 * (y-coordinate - $min-y) div ($max-y - $min-y)) ),
								' ')"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="$y-axis-justification='10-percent-normalize'">
					<xsl:choose>
						<xsl:when test="$max-y = $min-y">
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.5) ),
								' ')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.82 - 0.64 * (y-coordinate - $min-y) div ($max-y - $min-y)) ),
								' ')"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:when test="$y-axis-justification='10-percent-normalize-xy'">
					<xsl:choose>
						<xsl:when test="$max-y = $min-y">
							<xsl:value-of select="concat(
								ceiling( $width * (0.18 + 0.64 *  (x-coordinate - $min-x) div ($max-x - $min-x)) ),
								',',
								ceiling( $height * (0.5) ),
								' ')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat(
								ceiling( $width * (0.18 + 0.64 *  (x-coordinate - $min-x) div ($max-x - $min-x)) ),
								',',
								ceiling( $height * (0.82 - 0.64 * (y-coordinate - $min-y) div ($max-y - $min-y)) ),
								' ')"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:choose>
						<xsl:when test="$max-y = $min-y">
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.9) ),
								' ')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat(
								ceiling( $width * (0.1 + 0.8 *  x-coordinate div $max-x) ),
								',',
								ceiling( $height * (0.9 - 0.8 * y-coordinate div $max-y) ),
								' ')"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:variable>
	<!--TODO : externalize this param by using radar-config-->
    <!--<polyline fill="none" stroke="{$main-graph-line-color}" stroke-width="5" style="{$style}"-->
    <polyline fill="none" stroke="black" stroke-width="5" style="{$style}"
            points="{$points-series}"/>

</xsl:template>


<!-- plots a polyline on the graph -->
<xsl:template name="draw-y-axis">
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="max-x"/>
	<xsl:param name="min-x"/>
	<xsl:param name="max-y"/>
	<xsl:param name="min-y"/>
	<xsl:param name="y-axis-justification"/>

	  <g stroke="black" >
	    <line x1="{$width * 0.09}" y1="{$height * 0.9}" x2="{$width * 0.11}" y2="{$height * 0.9}"
	            stroke-width="4"/>
	    <line x1="{$width * 0.09}" y1="{$height * 0.74}" x2="{$width * 0.11}" y2="{$height * 0.74}"
	            stroke-width="4"/>
	    <line x1="{$width * 0.09}" y1="{$height * 0.58}" x2="{$width * 0.11}" y2="{$height * 0.58}"
	            stroke-width="4"/>
	    <line x1="{$width * 0.09}" y1="{$height * 0.42}" x2="{$width * 0.11}" y2="{$height * 0.42}"
	            stroke-width="4"/>
	    <line x1="{$width * 0.09}" y1="{$height * 0.26}" x2="{$width * 0.11}" y2="{$height * 0.26}"
	            stroke-width="4"/>
	    <line x1="{$width * 0.09}" y1="{$height * 0.1}" x2="{$width * 0.11}" y2="{$height * 0.1}"
	            stroke-width="4"/>
	  </g>

  	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="0"/>
	</xsl:call-template>

	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="1"/>
	</xsl:call-template>

	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="2"/>
	</xsl:call-template>

	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="3"/>
	</xsl:call-template>

	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="4"/>
	</xsl:call-template>

	<xsl:call-template name="y-axis-text-labels">
		<xsl:with-param name="width" select="$width"/>
		<xsl:with-param name="height" select="$height"/>
		<xsl:with-param name="max-x" select="$max-x"/>
		<xsl:with-param name="min-x" select="$min-x"/>
		<xsl:with-param name="max-y" select="$max-y"/>
		<xsl:with-param name="min-y" select="$min-y"/>
		<xsl:with-param name="y-axis-justification" select="$y-axis-justification"/>
		<xsl:with-param name="element-number" select="5"/>
	</xsl:call-template>
</xsl:template>

<xsl:template name="y-axis-text-labels">
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="max-x"/>
	<xsl:param name="min-x"/>
	<xsl:param name="max-y"/>
	<xsl:param name="min-y"/>
	<xsl:param name="y-axis-justification"/>
	<xsl:param name="element-number"/>

	<text font-family="Verdana"
		fill="{$grid-point-text-color}"
		x="{$width * 0.01}"
		y="{$height * (0.9 - 0.16 * $element-number)}"
		font-size="{ceiling($width * 0.02)}">
		<xsl:choose>
			<xsl:when test="$y-axis-justification='simple-normalize'">
			    	<xsl:choose>
			    		<xsl:when test="$max-y &gt; 1000">
			    			<xsl:value-of select="format-number($min-y + ($max-y - $min-y)* $element-number div 5,'#')"/>
			    		</xsl:when>
			    		<xsl:otherwise>
						    <xsl:value-of select="format-number($min-y + ($max-y - $min-y)* $element-number div 5,'#.##')"/>
						</xsl:otherwise>
					</xsl:choose>
			</xsl:when>
			<xsl:when test="$y-axis-justification='10-percent-normalize'">
			    	<xsl:choose>
			    		<xsl:when test="$max-y &gt; 1000">
			    			<xsl:value-of select="format-number($min-y - 0.1 * ($max-y - $min-y) + 1.2 * ($max-y - $min-y) * $element-number div 5,'#')"/>
			    		</xsl:when>
			    		<xsl:otherwise>
						    <xsl:value-of select="format-number($min-y - 0.1 * ($max-y - $min-y) + 1.2 * ($max-y - $min-y) * $element-number div 5,'#.##')"/>
						</xsl:otherwise>
					</xsl:choose>
			</xsl:when>
			<xsl:when test="$y-axis-justification='10-percent-normalize-xy'">
			    	<xsl:choose>
			    		<xsl:when test="$max-y &gt; 1000">
			    			<xsl:value-of select="format-number($min-y - 0.1 * ($max-y - $min-y) + 1.2 * ($max-y - $min-y) * $element-number div 5,'#')"/>
			    		</xsl:when>
			    		<xsl:otherwise>
						    <xsl:value-of select="format-number($min-y - 0.1 * ($max-y - $min-y) + 1.2 * ($max-y - $min-y) * $element-number div 5,'#.##')"/>
						</xsl:otherwise>
					</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
			    	<xsl:choose>
			    		<xsl:when test="$max-y &gt; 1000">
			    			<xsl:value-of select="format-number($max-y * $element-number div 5,'#')"/>
			    		</xsl:when>
			    		<xsl:otherwise>
						    <xsl:value-of select="format-number($max-y * $element-number div 5,'#.##')"/>
						</xsl:otherwise>
					</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</text>
</xsl:template>


<!-- plots a polyline on the graph -->
<xsl:template name="draw-x-axis">
	<xsl:param name="width"/>
	<xsl:param name="height"/>
	<xsl:param name="pointset"/>
	<xsl:param name="max-x"/>
	<xsl:param name="min-x"/>
	<xsl:param name="max-y"/>
	<xsl:param name="min-y"/>
	<xsl:param name="y-axis-justification"/>

    <xsl:for-each select="$pointset/points/point">
    	<xsl:sort select="x-coordinate" order="descending" data-type="number"/>
	    <text font-family="Verdana"
			fill="{$grid-point-text-color}"
			x="{$width * (0.08 + 0.8 *  x-coordinate div $max-x)}"
			y="{$height * 0.94}"
			font-size="{ceiling($width * 0.02)}">
				<xsl:value-of select="x-label"/>
		</text>
	</xsl:for-each>

</xsl:template>

</xsl:stylesheet>
