<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

    <xsl:import href="xradar://stylesheets/radar/graph/common-scripts.xsl"/>

        <xsl:param name="title-color" select="'$color-ko'"/>
    <xsl:param name="unit-color" select="$color-ok"/>
    <xsl:param name="dot-color" select="$color-ok"/>

<xsl:template name="relations-matrix-graph">
    <xsl:param name="title"/>
  	<xsl:param name="width"/>
  	<xsl:param name="height"/>
    <xsl:param name="relations-nodeset"/>
    <xsl:param name="units"/>
	<xsl:param name="dsm" select="'false'"/>

    <svg viewBox="0 0 {$width} {$height}" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">

        <xsl:variable name="relations" select="xalan:nodeset($relations-nodeset)"/>

        <xsl:call-template name="common-scripts"/>

          <!-- Show outline of canvas using 'rect' element -->
        <rect x="1" y="1" fill="none" style="stroke:{$title-color};stroke-width:4">
            <xsl:attribute name="width"><xsl:value-of select="$width - 2"/></xsl:attribute>
            <xsl:attribute name="height"><xsl:value-of select="$height - 2"/></xsl:attribute>
        </rect>

        <text font-family="Verdana" fill="{$title-color}" x="{$width * 0.98}" y="{$height * 0.98}">
            <xsl:attribute name="font-size">
                <xsl:value-of select="ceiling($width * 0.04)"/>
            </xsl:attribute>
            <xsl:attribute name="style">text-anchor:end;</xsl:attribute>
            <xsl:value-of select="$title"/>
        </text>

        <xsl:apply-templates select="$relations/relations" mode="set-up-matrix-frame">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="units" select="$units"/>
			<xsl:with-param name="dsm" select="$dsm"/>
        </xsl:apply-templates>

    </svg>
</xsl:template>


<xsl:template match="relations" mode="set-up-matrix-frame">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="units"/>
    <xsl:param name="dsm"/>

	<xsl:variable name="total-modules" select="count(descendant::module[@size &gt; 0])"/>
    <xsl:variable name="total-module-size" select="sum(descendant::module/@size)"/>
    <xsl:variable name="max-relation-width" select="0.15 * $width div $total-modules"/>
    <xsl:variable name="max-relation-weight">
        <xsl:for-each select="descendant::element">
            <xsl:sort select="@weight" data-type="number" order="descending"/>
            <xsl:if test="position() = 1">
                <xsl:value-of select="@weight"/>
            </xsl:if>
        </xsl:for-each>
    </xsl:variable>


    <rect x="{$width*0.1}" y="{$height*0.1}" style="fill:none;stroke:black;stroke-width:4" width="{$width - $width*0.2}" height="{$height - $height*0.2}"/>

    <xsl:for-each select="module[@size &gt; 0]">
        <xsl:variable name="module-position" select="position()"/>
        <xsl:variable name="size-up-to-last" select="sum(../module[position() &lt; $module-position]/@size)"/>
        <xsl:variable name="size-up-to-this" select="sum(../module[position() &lt;= $module-position]/@size)"/>


        <xsl:call-template name="matrix-rows-and-columns">
            <xsl:with-param name="width" select="$width"/>
            <xsl:with-param name="height" select="$height"/>
            <xsl:with-param name="total-module-size" select="$total-module-size"/>
            <xsl:with-param name="total-modules" select="$total-modules"/>
            <xsl:with-param name="module-position" select="$module-position"/>
            <xsl:with-param name="size-up-to-last" select="$size-up-to-last"/>
            <xsl:with-param name="size-up-to-this" select="$size-up-to-this"/>
            <xsl:with-param name="module-name" select="@name"/>
        </xsl:call-template>

        <xsl:for-each select="group">
            <xsl:apply-templates select="current()" mode="relation-markings">
                <xsl:with-param name="width" select="$width"/>
                <xsl:with-param name="height" select="$height"/>
                <xsl:with-param name="total-module-size" select="$total-module-size"/>
                <xsl:with-param name="total-modules" select="$total-modules"/>
                <xsl:with-param name="module-position" select="$module-position"/>
                <xsl:with-param name="size-up-to-this" select="$size-up-to-this"/>
                <xsl:with-param name="group-position" select="position()"/>
                <xsl:with-param name="max-relation-width" select="$max-relation-width"/>
                <xsl:with-param name="max-relation-weight" select="$max-relation-weight"/>
                <xsl:with-param name="units" select="$units"/>
                <xsl:with-param name="dsm" select="$dsm"/>
            </xsl:apply-templates>
        </xsl:for-each>

    </xsl:for-each>


</xsl:template>

<xsl:template name="matrix-rows-and-columns">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="total-module-size"/>
    <xsl:param name="total-modules"/>
    <xsl:param name="module-position"/>
    <xsl:param name="size-up-to-last"/>
    <xsl:param name="size-up-to-this"/>
    <xsl:param name="module-name"/>

    <xsl:variable name="adjustment-factor" select="0.10 + 0.8 * $size-up-to-this div $total-module-size"/>

    <!-- Horisontal Axis-->
    <line style="stroke:black;stroke-width:8">
            <xsl:attribute name="x1"><xsl:value-of select="ceiling($width * 0.10)"/></xsl:attribute>
            <xsl:attribute name="x2"><xsl:value-of select="ceiling($width * 0.90)"/></xsl:attribute>
            <xsl:attribute name="y1"><xsl:value-of select="ceiling($height * $adjustment-factor)"/></xsl:attribute>
            <xsl:attribute name="y2"><xsl:value-of select="ceiling($height * $adjustment-factor)"/></xsl:attribute>
    </line>



    <!-- Vertical Axis-->
    <line style="stroke:black;stroke-width:8">
        <xsl:attribute name="x1"><xsl:value-of select="ceiling($width * $adjustment-factor)"/></xsl:attribute>
        <xsl:attribute name="x2"><xsl:value-of select="ceiling($width * $adjustment-factor)"/></xsl:attribute>
        <xsl:attribute name="y1"><xsl:value-of select="ceiling($height * 0.10)"/></xsl:attribute>
        <xsl:attribute name="y2"><xsl:value-of select="ceiling($height * 0.90)"/></xsl:attribute>
    </line>

	<xsl:variable name="heading-x" select="ceiling($width * ($adjustment-factor - 0.4 * ($size-up-to-this - $size-up-to-last) div $total-module-size))"/>
	<xsl:variable name="heading-y" select="ceiling($height * 0.09)"/>
	<g  transform="translate({$heading-x}, {$heading-y}) rotate(-45) translate(-{$heading-x}, -{$heading-y})">
	    <text font-family="Verdana">
            <xsl:attribute name="fill">
                <xsl:value-of select="$color-ok"/>
            </xsl:attribute>
	        <xsl:attribute name="font-size">
	            <xsl:value-of select="ceiling($width * 0.015)"/>
	        </xsl:attribute>
	        <xsl:attribute name="style">text-anchor:start;</xsl:attribute>
	        <xsl:attribute name="x">
	            <xsl:value-of select="$heading-x"/>
	        </xsl:attribute>
	        <xsl:attribute name="y">
	        	<xsl:value-of select="$heading-y"/>
	        </xsl:attribute>
	        <xsl:value-of select="$module-name"/>
	    </text>
    </g>

    <g transform="translate({$heading-y}, {$heading-x}) rotate(-45) translate(-{$heading-y}, -{$heading-x})">
    <text font-family="Verdana" fill="{$unit-color}">
        <xsl:attribute name="font-size">
            <xsl:value-of select="ceiling($width * 0.015)"/>
        </xsl:attribute>
        <xsl:attribute name="style">text-anchor:end;</xsl:attribute>
        <xsl:attribute name="x">
            <xsl:value-of select="$heading-y"/>
        </xsl:attribute>
        <xsl:attribute name="y">
        	<xsl:value-of select="$heading-x"/>
        </xsl:attribute>
        <xsl:value-of select="$module-name"/>
    </text>
    </g>

</xsl:template>

<xsl:template match="group" mode="relation-markings">
    <xsl:param name="width"/>
    <xsl:param name="height"/>
    <xsl:param name="total-module-size"/>
    <xsl:param name="total-modules"/>
    <xsl:param name="module-position"/>
    <xsl:param name="size-up-to-this"/>
    <xsl:param name="group-position"/>
    <xsl:param name="max-relation-width"/>
    <xsl:param name="max-relation-weight"/>
    <xsl:param name="units"/>
    <xsl:param name="dsm"/>

    <xsl:variable name="color" select="concat($module-position,':',$group-position)"/>

    <xsl:variable name="internal-class-adjustment-factor" select="0.10 + 0.8 * ($size-up-to-this - ../@size) div $total-module-size"/>



		<xsl:if test="element[1]/@internal-relation = 'true'">
			<xsl:variable name="id" select="concat($module-position,':',$group-position)"/>
            <a xlink:href="{link}">
			<g onmouseover="onEvent(evt, '{$id}', 'visible');" onmouseout="onEvent(evt, '{$id}', 'hidden');">
				<xsl:choose>
				<xsl:when test="$dsm='false'">
					<circle style="fill:{$dot-color};stroke:{$dot-color};stroke-width:4">
						<xsl:attribute name="r">
							<xsl:value-of select="ceiling($max-relation-width * element[1]/@weight div $max-relation-weight)"/>
						</xsl:attribute>
						<xsl:attribute name="cx">
							<xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id - 0.5) div ($total-module-size)))"/>
						</xsl:attribute>
						<xsl:attribute name="cy">
							<xsl:value-of select="ceiling($height * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id - 0.5) div ($total-module-size)))"/>
						</xsl:attribute>
					</circle>
				</xsl:when>
				<xsl:otherwise>
					<line style="stroke:{$title-color};stroke-width:4">
						<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * ($internal-class-adjustment-factor+ 0.8 * (element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
					</line>
					<line style="stroke:{$title-color};stroke-width:4">
						<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor+ 0.8 * (element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
						<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * ($internal-class-adjustment-factor + 0.8 * (element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
					</line>
				</xsl:otherwise>
				</xsl:choose>
				</g>
				</a>
        </xsl:if>
        <xsl:for-each select="element">
			<xsl:if test="@source = 'false'">
            	<xsl:variable name="this-module-seq-number" select="@module-seq-number"/>
				<xsl:variable name="external-class-adjustment-factor" select="0.10 + 0.8 * sum(ancestor::relations/module[@seq-number &lt; $this-module-seq-number]/@size) div $total-module-size"/>
				<xsl:variable name="id" select="concat($module-position,':',$group-position,':',position())"/>
				<a xlink:href="{../link}">
				<g onmouseover="onEvent(evt, '{$id}', 'visible');" onmouseout="onEvent(evt, '{$id}', 'hidden');">
					<xsl:choose>
					<xsl:when test="$dsm='false'">
						<circle style="fill:{$dot-color};stroke:{$dot-color};stroke-width:4">
							<xsl:attribute name="r">
								<xsl:value-of select="ceiling($max-relation-width * @weight div $max-relation-weight)"/>
							</xsl:attribute>
							<xsl:attribute name="cx">
								<xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (../element[1]/@seq-id - 0.5) div ($total-module-size)))"/>
							</xsl:attribute>
							<xsl:attribute name="cy">
								<xsl:value-of select="ceiling($height * ($external-class-adjustment-factor + 0.8 * (@seq-id - 0.5) div ($total-module-size)))"/>
							</xsl:attribute>
						</circle>
					</xsl:when>
					<xsl:otherwise>
						<xsl:variable name="cross-color">
							<xsl:choose>
								<xsl:when test="@legal = 'true'"><xsl:value-of select="$title-color"/></xsl:when>
								<xsl:otherwise><xsl:value-of select="$dot-color"/></xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<line style="stroke:{$cross-color};stroke-width:4">
							<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (../element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (../element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * ($external-class-adjustment-factor+ 0.8 * (@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * ($external-class-adjustment-factor + 0.8 * (@seq-id) div ($total-module-size)))"/></xsl:attribute>
						</line>
						<line style="stroke:{$cross-color};stroke-width:4">
							<xsl:attribute name="x1"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor+ 0.8 * (../element[1]/@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="x2"><xsl:value-of select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (../element[1]/@seq-id) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="y1"><xsl:value-of select="ceiling($height * ($external-class-adjustment-factor + 0.8 * (@seq-id) div ($total-module-size)))"/></xsl:attribute>
							<xsl:attribute name="y2"><xsl:value-of select="ceiling($height * ($external-class-adjustment-factor + 0.8 * (@seq-id - 1) div ($total-module-size)))"/></xsl:attribute>
						</line>
					</xsl:otherwise>
					</xsl:choose>
				</g>
				</a>
			</xsl:if>
        </xsl:for-each>



        <xsl:for-each select="element">
			<g style="visibility:hidden;">
	        <xsl:attribute name="id"><xsl:choose>
				<xsl:when test="@source = 'false'"><xsl:value-of select="concat($module-position,':',$group-position,':',position())"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat($module-position,':',$group-position)"/></xsl:otherwise>
			</xsl:choose></xsl:attribute>

			<xsl:variable name="this-module-seq-number" select="@module-seq-number"/>
            <xsl:variable name="external-class-adjustment-factor" select="0.10 + 0.8 * sum(ancestor::relations/module[@seq-number &lt; $this-module-seq-number]/@size) div $total-module-size"/>
            <xsl:variable name="cirle-x" select="ceiling($width * ($internal-class-adjustment-factor + 0.8 * (../element[1]/@seq-id - 0.5) div ($total-module-size)))"/>
            <xsl:variable name="cirle-y" select="ceiling($height * ($external-class-adjustment-factor + 0.8 * (@seq-id - 0.5) div ($total-module-size)))"/>
			<xsl:variable name="x">
				<xsl:choose>
					<xsl:when test="$module-position div $total-modules &lt;  0.5"><xsl:value-of select="$cirle-x + $width * 0.05"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$cirle-x - $width * 0.05"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="y" select="$cirle-y - $height*0.02"/>
			<xsl:variable name="text-anchor">
				<xsl:choose>
					<xsl:when test="$module-position div $total-modules &lt;  0.5">start</xsl:when>
					<xsl:otherwise>end</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

            <circle
                style="fill:none;stroke:black;stroke-width:4;">
                <xsl:attribute name="r">
                    <xsl:value-of select="0.020*$width"/>
                </xsl:attribute>
                <xsl:attribute name="cx">
                    <xsl:value-of select="$cirle-x"/>
                </xsl:attribute>
                <xsl:attribute name="cy">
                    <xsl:value-of select="$cirle-y"/>
                </xsl:attribute>
            </circle>

            <text style="font-size:{ceiling($width * 0.015)};font-family:Verdana;fill:black;text-anchor:{$text-anchor}">
                <xsl:attribute name="x">
                    <xsl:value-of select="$x"/>
                </xsl:attribute>
                <xsl:attribute name="y">
                    <xsl:value-of select="$y + $height * 0.02"/>
                </xsl:attribute>
                <xsl:value-of select="../element[@source = 'true']/@name"/>
            </text>

            <text style="font-size:{ceiling($width * 0.015)};font-family:Verdana;fill:black;text-anchor:{$text-anchor}">
                <xsl:attribute name="x">
                    <xsl:value-of select="$x"/>
                </xsl:attribute>
                <xsl:attribute name="y">
                    <xsl:value-of select="$y + $height * 0.04"/>
                </xsl:attribute>
                <xsl:value-of select="concat('-->',@name)"/>
				<xsl:if test="@weight &gt; 1">(<xsl:value-of select="@weight"/>)</xsl:if>
            </text>
			</g>
        </xsl:for-each>


</xsl:template>

</xsl:stylesheet>
